import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:ui';

import 'package:advertising_id/advertising_id.dart';
import 'package:appsflyer_sdk/appsflyer_sdk.dart';
import 'package:device_info/device_info.dart';
import 'package:firebase_analytics/firebase_analytics.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:http/http.dart' as http;
import 'package:intl/intl.dart';
import 'package:lemnisk_plugin/AttributeBuilder.dart';
import 'package:package_info_plus/package_info_plus.dart';
import 'package:platform_device_id_platform_interface/platform_device_id_platform_interface.dart';
import 'package:uuid/uuid.dart';

import 'LemniskFlutterCore.dart';
import 'firebase/LemniskFirebase.dart';
import 'model/GenricModel.dart';

class LemniskHelper {
  static FirebaseApp? _appFirebase;
  static AppsflyerSdk? _appsflyerSdk;

  static String? _writeKey;
  static String? _serverUrl;

  //firebase App Core
  static bool? _enableFirebase;
  static String? _projectName;
  static String? _appId;
  static String? _apiKey;
  static String? _projectId;
  static String? _messagingSenderId;

  static bool? _enableAppsFlyer;
  static String? _afDevKey;
  static String? _afAppId;
  static FirebaseAnalytics? analytics;

  /*static LemniskHelper? lemniskHelper;

  static LemniskHelper? getInstance() {
    if(lemniskHelper == null) {
      return new LemniskHelper();
    }
    return lemniskHelper;
  }*/

  static Future<String?> configure(String? serverUrl, String? writeKey, LemniskFirebaseCore? firebase, LemniskApsflyerCore? apsflyer) async {
    WidgetsFlutterBinding.ensureInitialized();
    //lemnisk server...
    _writeKey = writeKey;
    _serverUrl = serverUrl;

    //appsflyer
    _enableAppsFlyer = apsflyer!.enable;
    _afDevKey = apsflyer.afDevKey;
    _afAppId = apsflyer.afAppId;

    if (_enableAppsFlyer == true) {
      try{
      Map appsFlyerOptions = {
        "afDevKey": _afDevKey,
        "afAppId": _afAppId,
        "isDebug": true,
      };

      _appsflyerSdk = AppsflyerSdk(appsFlyerOptions);
      _appsflyerSdk?.initSdk(
        registerConversionDataCallback: true,
        registerOnAppOpenAttributionCallback: true,
        registerOnDeepLinkingCallback: true,
      );
      } on Exception catch (e) {
        print("Lemnisk AppsFlyer initialization Error: ${e.toString()}");
      }
      print('Lemnisk AppsFlyer Enabled');
    } else {
      print('Lemnisk AppsFlyer Disabled');
    }

    //firebase core....
    _enableFirebase = firebase!.enable;
    _projectName = firebase.projectName;
    _appId = firebase.appId;
    _apiKey = firebase.apiKey;
    _projectId = firebase.projectId;
    _messagingSenderId = firebase.messagingSenderId;

    if (_enableFirebase == true) {
      try {
        FirebaseOptions firebaseOptions = FirebaseOptions(
          appId: _appId!,
          apiKey: _apiKey!,
          projectId: _projectId!,
          messagingSenderId: _messagingSenderId!
        );
        _appFirebase = await Firebase.initializeApp(name: _projectName!, options: firebaseOptions);
        analytics = FirebaseAnalytics.instance;
      } on Exception catch (e) {
        print("Lemnisk Firebase intialization Error: ${e.toString()}");
      }
      print('Initialization Lemnisk Firebase Enabled');
    } else {
      print('Lemnisk Firebase Disabled');
    }
  }

  static final DeviceInfoPlugin _deviceInfoPlugin = DeviceInfoPlugin();
  //get Device ID
  static Future<String?> get _getDeviceId async {
    String? deviceId;
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await _deviceInfoPlugin.androidInfo;
        deviceId = androidInfo.androidId;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await _deviceInfoPlugin.iosInfo;
        deviceId = iosInfo.identifierForVendor;
      } else {
        deviceId = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceId = '';
    }
    return deviceId;
  }

  //get Plateform Device Type
  static Future<String?> get _getPlatFormDeviceType async {
    String? deviceType;
    try {
      if (Platform.isAndroid) {
        deviceType = "Android";
      } else if (Platform.isIOS) {
        deviceType = "IOS";
      }
    } on PlatformException {
      deviceType = '';
    }
    return deviceType;
  }

  //get Device Model Name
  static Future<String?> get _getDeviceModel async {
    String? deviceModel = '';
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await _deviceInfoPlugin.androidInfo;
        deviceModel = androidInfo.model;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await _deviceInfoPlugin.iosInfo;
        deviceModel = iosInfo.name;
      } else {
        deviceModel = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceModel = '';
    }
    return deviceModel;
  }

  //get Device Manufacture Model
  static Future<String?> get _getManufactureModel async {
    String? deviceManufacture = '';
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await _deviceInfoPlugin.androidInfo;

        deviceManufacture = androidInfo.manufacturer;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await _deviceInfoPlugin.iosInfo;
        deviceManufacture = iosInfo.systemName;
      } else {
        deviceManufacture = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceManufacture = '';
    }
    return deviceManufacture;
  }

  //get Device OS Version
  static Future<String?> get _getOSVersion async {
    String? deviceID = '';
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await _deviceInfoPlugin.androidInfo;
        deviceID = androidInfo.version.release;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await _deviceInfoPlugin.iosInfo;
        deviceID = iosInfo.systemVersion;
      } else {
        deviceID = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceID = '';
    }
    return deviceID;
  }

  //get Device Google Advertising ID
  static Future<String?> get _getGoogleadvertisingID async {
    String? advertisingId;
    try {
      if (Platform.isAndroid) {
        try {
          advertisingId = await AdvertisingId.id(true);
        } on PlatformException {
          advertisingId = 'Failed to get platform version.';
        }
      } else if (Platform.isIOS) {
        advertisingId = await AdvertisingId.id(true);
      } else {
        advertisingId = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      advertisingId = '';
    }
    return advertisingId;
  }

  //get Device Google Advertising bool Value
  static Future<bool?> get _getGoogleadvertisingIDBoolValue async {
    bool? isLimitAdTrackingEnabled;
    try {
      if (Platform.isAndroid) {
        try {
          isLimitAdTrackingEnabled = await AdvertisingId.isLimitAdTrackingEnabled;
        } on PlatformException {
          isLimitAdTrackingEnabled = false;
        }
      } else if (Platform.isIOS) {
        isLimitAdTrackingEnabled = await AdvertisingId.isLimitAdTrackingEnabled;
      }
    } on PlatformException {
      isLimitAdTrackingEnabled = false;
    }
    return isLimitAdTrackingEnabled;
  }

  //get Device Width
  static Future<String?> get _getScreenWidth async {
    var physicalScreenSize = window.physicalSize;
    var physicalWidth = physicalScreenSize.width;
    return physicalWidth.toString();
  }

  //get Device Height
  static Future<String> get _getScreenHeight async {
    var physicalScreenSize = window.physicalSize;
    var physicalHeight = physicalScreenSize.height;
    return physicalHeight.toString();
  }

  //get App Name
  static Future<String> get _getAppName async {
    PackageInfo _packageInfo = PackageInfo(
      appName: 'Unknown',
      packageName: 'Unknown',
      version: 'Unknown',
      buildNumber: 'Unknown',
      buildSignature: 'Unknown',
    );
    final info = await PackageInfo.fromPlatform();
    _packageInfo = info;
    return _packageInfo.appName;
  }

  //get App Version
  static Future<String> get _getAppVersion async {
    PackageInfo _packageInfo = PackageInfo(
      appName: 'Unknown',
      packageName: 'Unknown',
      version: 'Unknown',
      buildNumber: 'Unknown',
      buildSignature: 'Unknown',
    );
    final info = await PackageInfo.fromPlatform();
    _packageInfo = info;
    return _packageInfo.version;
  }

  //get App Build No
  static Future<String> get _getBuildNumber async {
    PackageInfo _packageInfo = PackageInfo(
      appName: 'Unknown',
      packageName: 'Unknown',
      version: 'Unknown',
      buildNumber: 'Unknown',
      buildSignature: 'Unknown',
    );
    final info = await PackageInfo.fromPlatform();
    _packageInfo = info;
    return _packageInfo.buildNumber;
  }

  //get Device Network IP
  static Future<String?> get _getNetworkIP async {
    final url = Uri.parse('https://api.ipify.org/');
    final response = await http.get(url);
    return response.body;
  }

  //get Payload Received Time
  static Future<dynamic> get _getReceivedAt async {
    return DateTime.now().toUtc().toString();
  }

  //get Device Payload Send Time
  static Future<String?> get _getSendAt async {
    DateTime getCurrentTimestamp = DateTime.now();
    String agreementSigned = formatISOTime(getCurrentTimestamp);
    return agreementSigned;
  }

  //get Device TimesTemp
  static Future<String?> get _getTimestemp async {
    DateTime getCurrentTimestamp = DateTime.now();
    String agreementSigned = formatISOTime(getCurrentTimestamp);
    return agreementSigned;
  }

  static String formatISOTime(DateTime date) {
    var duration = date.timeZoneOffset;
    if (duration.isNegative)
      return (DateFormat("yyyy-MM-ddTHH:mm:ss.mmm").format(date) + "-${duration.inHours.toString().padLeft(2, '0')}${(duration.inMinutes - (duration.inHours * 60)).toString().padLeft(2, '0')}");
    else
      return (DateFormat("yyyy-MM-ddTHH:mm:ss.mmm").format(date) + "+${duration.inHours.toString().padLeft(2, '0')}${(duration.inMinutes - (duration.inHours * 60)).toString().padLeft(2, '0')}");
  }

  //get Device Original TimesTemp
  static Future<String?> get _getOriginalTimestemp async {
    DateTime getCurrentTimestamp = DateTime.now();
    String agreementSigned = formatISOTime(getCurrentTimestamp);
    return agreementSigned;
  }

  //get Device Original TimesTemp
  static Future<String?> get _getServerTs async {
    return DateTime.now().toUtc().toString();
  }

  //get Device Original TimesTemp
  static Future<String?> get _getMessageId async {
    var uuid = Uuid();
    return uuid.v4();
  }

  static void track(String name, AttributeBuilder properties, OtherBuilder otherid) async {
    GenricModel genricModel = GenricModel();
    genricModel.name = name;
    genricModel.properties = properties.properties;
    genricModel.otherid = otherid.otherid;
    GenricModel? fMain = await genricModel;

    String? deviceId;
    String? deviceType;
    String? deviceManufacture;
    String? deviceModel;
    String? versionName;

    String? appName;
    String? appVersion;
    String? appBuild;

    String? googleAdvertisingID;
    dynamic? googleAdvertisingBool;

    String? networkIP;

    String? appScreenHeightWidth;
    String? appScreenHeightHeight;

    String? appDataReceivedAt;
    String? appDataSenddAt;
    String? appOriginalTimetemp;
    String? appTimestemp;
    String? appServerTs;
    String? appMsgId;

    try {
      deviceId = await _getDeviceId;
      deviceType = await _getPlatFormDeviceType;
      deviceModel = await _getDeviceModel;
      deviceManufacture = await _getManufactureModel;
      versionName = await _getOSVersion;
      appName = await _getAppName;
      appVersion = await _getAppVersion;
      appBuild = await _getBuildNumber;
      googleAdvertisingID = await _getGoogleadvertisingID;
      googleAdvertisingBool = await _getGoogleadvertisingIDBoolValue;
      networkIP = await _getNetworkIP;

      appScreenHeightHeight = await _getScreenHeight;
      appScreenHeightWidth = await _getScreenWidth;

      appDataReceivedAt = await _getReceivedAt;
      appDataSenddAt = await _getSendAt;

      appOriginalTimetemp = await _getOriginalTimestemp;
      appServerTs = await _getServerTs;
      appTimestemp = await _getTimestemp;
      appMsgId = await _getMessageId;
    } on PlatformException {
      deviceId = 'Failed to get information.';
    }

    Map<dynamic, dynamic> lib = {
      "library": {
        "name": "Flutter SDK",
        "version": "0.0.7",
      },
      "server_ts": appServerTs,
      "app": {
        "name": appName,
        "version": appVersion,
        "build": appBuild,
      },
      "device": {
        "id": deviceId,
        "advertisingId": googleAdvertisingID,
        "adTrackingEnabled": googleAdvertisingBool,
        "manufacturer": deviceManufacture,
        "model": deviceModel,
        "type": deviceType,
        "token": LemniskFirebase.FCM_URL,
      },
      "screen": {
        "width": appScreenHeightWidth,
        "height": appScreenHeightHeight,
        "density": 2,
      },
      "userAgent": {
        "osType": deviceType,
        "osVersion": versionName,
      },
      "ip": networkIP
    };
    Map map = {
      "id": deviceId,
      "messageId": appMsgId,
      "userId": "userId",
      "otherIds": fMain.otherid,
      "event": name,
      "properties": fMain.properties,
      "receivedAt": appDataReceivedAt,
      "sentAt": appDataSenddAt,
      "timestamp": appTimestemp,
      "type": "track",
      "originalTimestamp": appOriginalTimetemp,
      "writeKey": _writeKey,
    };
    map.addAll({"context": lib});

    //After the full payload is created..now we send all to lemnisk server
    print('track event' + _serverUrl!);
    Uri uri = Uri.parse(_serverUrl!);
    var response = await http.post(uri, body: JsonEncoder().convert(map));
    if (response.statusCode == 200) {
      final String responseString = response.body;
      print('event track success =====   $responseString');
    } else {
      final String responseString = response.body;
      print('event track failed =====   $responseString');
    }

    //appsflyer analytics
    if (_enableAppsFlyer == true) {
      bool? result;
      try {
        String apfName = name;
        if (name == 'signIn') {
          apfName = "af_login";
        } else if (name == 'signUp') {
          apfName = "af_complete_registration";
        }

        result = await _appsflyerSdk?.logEvent(apfName, properties.properties);

        if (result == true) {
          print("Result logEvent appsflyer success: ${result}");
        } else {
          print("Result logEvent appsflyer failed: ${result}");
        }
      } on Exception catch (e) {
        print("Result logEvent appsflyer Error: ${e.toString()}");
      }
    }

    if (_enableFirebase == true) {
      //firebase analytics
      String gaName = name;
      switch(name) {
        case "signUp":
          gaName = "sign_up";
          break;
        case "signIn":
          gaName = "login";
          break;
        case "searchSubmit":
          gaName = "search";
          break;
        case "serviceListViewed":
          gaName = "view_item_list";
          break;
        case "serviceDetailViewed":
          gaName = "view_item";
          break;
        case "serviceClicked":
          gaName = "select_item";
          break;
        case "bookServiceClicked":
          gaName = "begin_checkout";
          break;
        case "bookingCompleted":
          gaName = "purchase";
          break;
        case "skipSignIn":
          gaName = "sign_up";
          break;
        case "submitMobileNumber":
          gaName = "sign_up";
          break;
        case "bookServicePayLaterClicked":
          gaName = "begin_checkout";
          break;
        default:
          break;
      }
      analytics?.logEvent(name: gaName, parameters: properties.properties);
      print("Result logEvent firebase");
    }
  }

  static void identify(String userID, AttributeBuilder properties, OtherBuilder otherid) async {
    GenricModel genricModel = GenricModel();
    genricModel.name = userID;
    genricModel.properties = properties.properties;
    genricModel.otherid = otherid.otherid;
    GenricModel? fMain = await genricModel;

    String? deviceId;
    String? deviceType;
    String? deviceManufacture;
    String? deviceModel;
    String? versionName;

    String? appName;
    String? appVersion;
    String? appBuild;

    String? googleAdvertisingID;
    dynamic? googleAdvertisingBool;

    String? networkIP;

    String? appScreenHeightWidth;
    String? appScreenHeightHeight;

    String? appDataReceivedAt;
    String? appDataSenddAt;
    String? appOriginalTimetemp;
    String? appTimestemp;
    String? appServerTs;
    String? appMsgId;

    try {
      deviceId = await _getDeviceId;
      deviceType = await _getPlatFormDeviceType;
      deviceModel = await _getDeviceModel;
      deviceManufacture = await _getManufactureModel;
      versionName = await _getOSVersion;
      appName = await _getAppName;
      appVersion = await _getAppVersion;
      appBuild = await _getBuildNumber;
      googleAdvertisingID = await _getGoogleadvertisingID;
      googleAdvertisingBool = await _getGoogleadvertisingIDBoolValue;
      networkIP = await _getNetworkIP;

      appScreenHeightHeight = await _getScreenHeight;
      appScreenHeightWidth = await _getScreenWidth;

      appDataReceivedAt = await _getReceivedAt;
      appDataSenddAt = await _getSendAt;

      appOriginalTimetemp = await _getOriginalTimestemp;
      appTimestemp = await _getTimestemp;
      appServerTs = await _getServerTs;
      appMsgId = await _getMessageId;
    } on PlatformException {
      deviceId = 'Failed to get information.';
    }

    Map<dynamic, dynamic> lib = {
      "library": {
        "name": "Flutter SDK",
        "version": "0.0.7",
      },
      "server_ts": appServerTs,
      "app": {
        "name": appName,
        "version": appVersion,
        "build": appBuild,
      },
      "device": {
        "id": deviceId,
        "advertisingId": googleAdvertisingID,
        "adTrackingEnabled": googleAdvertisingBool,
        "manufacturer": deviceManufacture,
        "model": deviceModel,
        "type": deviceType,
        "token": LemniskFirebase.FCM_URL,
      },
      "screen": {
        "width": appScreenHeightWidth,
        "height": appScreenHeightHeight,
        "density": 2,
      },
      "userAgent": {
        "osType": deviceType,
        "osVersion": versionName,
      },
      "ip": networkIP
    };
    Map map = {
      "id": deviceId,
      "userId": "userId",
      "otherIds": fMain.otherid,
      "event": userID,
      "messageId": appMsgId,
      "customerProperties": fMain.properties,
      "receivedAt": appDataReceivedAt,
      "sentAt": appDataSenddAt,
      "timestamp": appTimestemp,
      "type": "identify",
      "originalTimestamp": appOriginalTimetemp,
      "writeKey": _writeKey,
    };
    map.addAll({"context": lib});

    //After the full payload is created..now we send all to lemnisk server
    print('identify event' + _serverUrl!);
    Uri uri = Uri.parse(_serverUrl!);
    var response = await http.post(uri, body: JsonEncoder().convert(map));
    if (response.statusCode == 200) {
      final String responseString = response.body;
      print('event identify success =====   $responseString');
    } else {
      final String responseString = response.body;
      print('event identify failed =====   $responseString');
    }

    if (_enableAppsFlyer == true) {
      try {
        _appsflyerSdk?.setCustomerUserId(userID);
        print('AppsFlyer userId set as $userID');
        // _appsflyerSdk?.setUserEmails(emails)
      } on Exception catch (e) {
        print("Result logEvent appsflyer Error: ${e.toString()}");
      }
    }

    if (_enableFirebase == true) {
      //firebase analytics
      analytics?.setUserId(id: userID);
      // analytics?.logScreenView()
      print("Result logEvent firebase");
    }
  }

  static void screen(String screenName, AttributeBuilder properties, OtherBuilder otherid) async {
    GenricModel genricModel = GenricModel();
    genricModel.name = screenName;
    genricModel.properties = properties.properties;
    genricModel.otherid = otherid.otherid;
    GenricModel? fMain = await genricModel;

    String? deviceId;
    String? deviceType;
    String? deviceManufacture;
    String? deviceModel;
    String? versionName;

    String? appName;
    String? appVersion;
    String? appBuild;

    String? googleAdvertisingID;
    dynamic? googleAdvertisingBool;

    String? networkIP;

    String? appScreenHeightWidth;
    String? appScreenHeightHeight;

    String? appDataReceivedAt;
    String? appDataSenddAt;
    String? appOriginalTimetemp;
    String? appTimestemp;
    String? appServerTs;
    String? appMsgId;

    try {
      deviceId = await _getDeviceId;
      deviceType = await _getPlatFormDeviceType;
      deviceModel = await _getDeviceModel;
      deviceManufacture = await _getManufactureModel;
      versionName = await _getOSVersion;
      appName = await _getAppName;
      appVersion = await _getAppVersion;
      appBuild = await _getBuildNumber;
      googleAdvertisingID = await _getGoogleadvertisingID;
      googleAdvertisingBool = await _getGoogleadvertisingIDBoolValue;
      networkIP = await _getNetworkIP;

      appScreenHeightHeight = await _getScreenHeight;
      appScreenHeightWidth = await _getScreenWidth;

      appDataReceivedAt = await _getReceivedAt;
      appDataSenddAt = await _getSendAt;

      appOriginalTimetemp = await _getOriginalTimestemp;
      appTimestemp = await _getTimestemp;
      appServerTs = await _getServerTs;
      appMsgId = await _getMessageId;
    } on PlatformException {
      deviceId = 'Failed to get information.';
    }

    Map<dynamic, dynamic> lib = {
      "library": {
        "name": "Flutter SDK",
        "version": "0.0.7",
      },
      "server_ts": appServerTs,
      "app": {
        "name": appName,
        "version": appVersion,
        "build": appBuild,
      },
      "device": {
        "id": deviceId,
        "advertisingId": googleAdvertisingID,
        "adTrackingEnabled": googleAdvertisingBool,
        "manufacturer": deviceManufacture,
        "model": deviceModel,
        "type": deviceType,
        "token": LemniskFirebase.FCM_URL,
      },
      "screen": {
        "width": appScreenHeightWidth,
        "height": appScreenHeightHeight,
        "density": 2,
      },
      "userAgent": {
        "osType": deviceType,
        "osVersion": versionName,
      },
      "ip": networkIP
    };

    Map map = {
      "id": deviceId,
      "userId": "userId",
      "otherIds": fMain.otherid,
      "event": screenName,
      "messageId": appMsgId,
      "properties": fMain.properties,
      "receivedAt": appDataReceivedAt,
      "sentAt": appDataSenddAt,
      "timestamp": appTimestemp,
      "type": "screen",
      "originalTimestamp": appOriginalTimetemp,
      "writeKey": _writeKey,
    };
    map.addAll({"context": lib});

    //After the full payload is created..now we send all to lemnisk server
    print('Screen event' + _serverUrl!);
    Uri uri = Uri.parse(_serverUrl!);
    var response = await http.post(uri, body: JsonEncoder().convert(map));
    if (response.statusCode == 200) {
      final String responseString = response.body;
      print('event Screen success =====   $responseString');
    } else {
      final String responseString = response.body;
      print('event Screen failed =====   $responseString');
    }

    //appsflyer analytics
    if (_enableAppsFlyer == true) {
      bool? result;
      try {
        result = await _appsflyerSdk?.logEvent("screen: $screenName", properties.properties);

        if (result == true) {
          print("Result logEvent appsflyer success: ${result}");
        } else {
          print("Result logEvent appsflyer failed: ${result}");
        }
      } on Exception catch (e) {
        print("Result logEvent appsflyer Error: ${e.toString()}");
      }
    }
  }

  /*static Map<String, dynamic> recurseAndPopulate(dynamic gaProp, Map<String, dynamic> properties) {
    Type type = properties[gaProp].runtimeType;
    var keys;
    if(type == Map) {
      keys = properties[gaProp].keys;
    }
    Map<String, dynamic> returnValue = {};
    for( var index in keys) {
      var value = gaProp[keys[index]];
      var updatedValue = "";
      // Type of value can be String or Array or Object
      if (value.runtimeType == String){
        updatedValue = getStringValue(value, properties);
      } else if (value.runtimeType == Map) {
        updatedValue = recurseAndPopulate(value, properties);
      } else if(value.runtimeType == List) {
        updatedValue = getArrayValue(value, properties);
      }
      returnValue[keys[index]] = updatedValue;
    }
    return returnValue;
  }

  static Array getArrayValue(value, properties) {
    var returnValue = [];
    for(var index in value) {
      var indexvalue = value[index];
      var updatedValue = "";
      if (typeof indexvalue == "string"){
        updatedValue = lmSMTObj.getStringValue(indexvalue, properties);
      } else if (typeof indexvalue == "object") {
        if(indexvalue.length){
          updatedValue = lmSMTObj.getArrayValue(indexvalue, properties);
        } else {
          updatedValue = lmSMTObj.recurseAndPopulate(indexvalue, properties);
        }
      }
      returnValue[index] = updatedValue;
    }
    return returnValue;
  }

  static String getStringValue(value, properties) {
    if(value.startsWith('properties')){
      var fields = value.split('.');
      return getValue(properties, fields.slice(1));
    }
    return value;
  }

  static Object getValue(properties, fields){
    if(fields && fields.length == 1 ){
      if (properties.hasOwnProperty(fields[0]))
        return properties[fields[0]];
      else return "";
    }
    var hasOwnprop = properties.hasOwnProperty(fields[0]);
    var typo = typeof properties[fields[0]] == "object";
    if( hasOwnprop && typo ){
      return getValue(properties[fields[0]], fields.slice(1));
    }
    return "";
  }*/
}
